#!/bin/ksh
#
# NAME:		cics_portable.fnc - Provide portability layer for shell scripts
#
# VERSION:	1.1
#
#                                                         
#   (C) COPYRIGHT International Business Machines Corp.   
#   2011                                            
#   All Rights Reserved                                   
#   Licensed Materials - Property of IBM                  
#   5725-C89                                              
#                                                         
#   US Government Users Restricted Rights -               
#   Use, duplication or disclosure restricted by          
#   GSA ADP Schedule Contract with IBM Corp.              
#                                                         
#
# SYNOPSIS:	
#
# CheckCanRun File ...
# $CICS_LOG_LV_RAW_PATH
# $CICS_LOG_LV_BLOCK_PATH
# $CICS_SFS_LV_RAW_PATH
# $CICS_SFS_LV_BLOCK_PATH
# $CICS_PPCGWY_LV_RAW_PATH
# $CICS_PPCGWY_LV_BLOCK_PATH
# $CICS_SIGTERM
# $CICS_SIGUSR1
# $CICS_SIGHUP
# $CICS_SIGINT
# 
# CompileAndLink
#
# ListLV Pathname
#
# GetServerName <sub-system_name>
#
# GetRegionName <sub-system_name>
#
# GetSubSysStdOut <sub-system_name>
#
# MapLVol <server> <path_to_raw_device>
#
# RgyEdit [<args>]
#
# RgyAddPrincipal [<args>]
#
# GetEUid
#
# KList
#
# $GETENCINAPATH
#
# DESCRIPTION:	
# This file contains all shell parameters and functions needed to provide
# a common and/or portable interface to implement any functionality used by
# the base CICS/6000 shell script code where there isn't native support for
# that function on all platforms.
#
# Where a native function is wrapped into a function in this file, the
# native function is aliased to a PortabilityTrap function. This prevents
# future coders from introducing calls to this non-portable native function.
#
# On the RS/6000 under AIX v3.2.4 support is required for:
#
#	- Checking for execute permission on programs
#	- Paths to filesystem nodes where logical volumes are kept
#	- A portable interface to listing logical volume entries
#       - A portable means to get the log/sfs server name for an SRC sub-system
#	- A portable means to create a logical volume
#	- A portable and consistent interface to rgy_edit
#
# CAVEATS/WARNINGS:
# None.
#		
# RETURNED VALUES:
# CheckCanRun returns 0 if one or more programs cannot be run, otherwise 1.
#
# ListLV returns 0 if the LV exists, otherwise 1.
#
# GetServerName returns 0 if all went ok, otherwise 1.
#
# MapLVol returns 0 if all went ok, otherwise >0.
#
# RgyEdit returns 0 if all went ok, otherwise >0.
#
# RELATED INFORMATION:
# None.
#
#
#*/

#------------------------------------------------------------------------
#
# SYNOPSIS: PortabilityTrap
# DESCRIPTION:
# All identified non-portable programs end up in here to trap coders 
# injecting porting errors which have already been addressed. They should
# use one of the facilities provided by this file instead.
#
alias odmget=PortabilityTrap
alias rgy_edit=PortabilityTrap
alias lslv=PortabilityTrap

PortabilityTrap()
{
    print -u2 "ASSERT: non-portable command called - see cicsportable.fnc"
    exit 1
}

#------------------------------------------------------------------------
#
# SYNOPSIS: GetEUid
# DESCRIPTION:
# Obtain the effective user id of the current user
#
#
alias GetEUid="id -u"

#------------------------------------------------------------------------
#
# SYNOPSIS: KList
# DESCRIPTION:
# Obtain the effective user id of the current user
#
#
alias KList="/usr/lpp/dce/bin/klist"

#------------------------------------------------------------------------
#
# SYNOPSIS: CompileAndLink
# DESCRIPTION:
# Perform a system specific compile and link step.
#

CompileAndLink()
{

	eval typeset CLType=$1	# type of processing required
	eval typeset CLLang=$2	# language being processed

	case $CLLang in
	C)	eval typeset 	CLCICS=$3	# CICS library
		eval typeset	CLUser=$4	# User library
		eval typeset	CLCC=$5		# CC flags
		eval typeset	CLFile=$6	# file name
		eval typeset	CLLD=$7		# link flags
		eval typeset 	CLIncl=$8	# include libraries

		CLCC=" -I${CLIncl} ${CLCC} -o ${CLFile}"
		CLCICS="-bI:${CLCICS}/cicsprC.exp -e main"
		COMPILE="xlc_r4 ${CLCICS} ${CLUser} ${CLCC} ${CLLD}"
		COMPILE="${COMPILE} ${CLFile}.c"
		;;
        IBMCPP)    eval typeset    CLCICS=$3       # CICS library
                eval typeset    CLUser=$4       # User library
                eval typeset    CLCC=$5         # CC flags
                eval typeset    CLFile=$6       # file name
                eval typeset    CLLD=$7         # link flags
                eval typeset    CLIncl=$8       # include libraries

                CLCC=" -I${CLIncl} ${CLCC} -c -o ${CLFile}.o"
                CLCICS="-bI:${CLCICS}/cicsprCpp.exp"
                CLCICS="${CLCICS} -lC_r"
                #
                # The following two lines ensure that the CICS application
                # is linked against the pthread draft 4 compatability libraries
                # as specified in the xlC_r4 stanza of /etc/xlC.cfg
                #
                CLCICS="${CLCICS} -L/usr/lib/dce"
                CLCICS="${CLCICS} -ldcelibc_r -ldcepthreads"
                COMPILE="xlC_r4 ${CLCC}"
                COMPILE="${COMPILE} ${CLFile}.C"
 		CLLD="-e main ${CLCICS} ${CLUser} ${CLLD}"
		#
		# makeC++SharedLib must be used with CSet++ to ensure
		# that static objects are initialised via loadandinit()
		# in the C++ language method
		#
		LINK="xlC_r4 -qmkshorbj -o ${CLFile}.ibmcpp ${CLLD}"
                LINK="${LINK} ${CLFile}.o"
                ;;
	COBOL)	eval typeset CLaOPT=$3		# animator option
		eval typeset CLCOBO=$4		# cobol flags
		eval typeset CLFile=$5		# file name
		COMPILE="cob ${CLaOPT} ${CLCOBO} ${CLFile}.cbl"
		;;
	IBMCOB)	eval typeset 	CLCICS=$3	# CICS library
		eval typeset	CLUser=$4	# User library
		eval typeset	CLFlags=$5	# IBMCOB flags
		eval typeset	CLFile=$6	# file name
		eval typeset	CLLD=$7		# link flags

		CLFlags=" ${CLFlags} -o ${CLFile}.ibmcob"
		#
		# Pull in the language method's exported symbols, and
		# ensure the correct entry point is used
		#
		CLCICS="-bI:${CLCICS}/cicsprIBMCOB.exp -e _iwz_cobol_main"
                #
 		# The following two lines ensure that the CICS application
		# is linked against the pthread draft 4 compatability libraries
		# as specified in the xlc_r4 stanza of /etc/xlC.cfg
		#
                CLCICS="${CLCICS} -L/usr/lib/dce"
                CLCICS="${CLCICS} -ldcelibc_r -ldcepthreads"
		#
		# Specify -qLIB to allow copybooks, and ensure that the
		# most appropriate stanza in cob2.cfg is used.
		#
		COMPILE="cob2_r -qLIB"
                COMPILE="${COMPILE} ${CLCICS} ${CLUser} ${CLFlags} ${CLLD}"
		COMPILE="${COMPILE} ${CLFile}.cbl"
		;;
        IBMPLI) eval typeset    CLCICS=$3       # CICS library
                eval typeset    CLUser=$4       # User library
                eval typeset    CLFlags=$5      # IBMPLI flags
                eval typeset    CLFile=$6       # file name
                eval typeset    CLLD=$7         # link flags
		eval typeset 	CLIncl=$8	# include libraries

                #
                # Obtain the base filename i.e. without path for output.
                # 
                FILENAME=$(basename ${CLFile})
                CLFlags=" -I${CLIncl} ${CLFlags} -o ${FILENAME%.pli}.ibmpli"
                #
                # Pull in the language method's exported symbols, and
                # ensure the correct entry point is used
                #
                CLCICS="-bI:${CLCICS}/cicsprIBMPLI.exp -eplicics"
                #
                # The following two lines ensure that the CICS application
                # is linked against the pthread draft 4 compatability libraries
                # as specified in the xlc_r4 stanza of /etc/xlC.cfg
                #
                CLCICS="${CLCICS} -L/usr/lib/dce"
                CLCICS="${CLCICS} -ldcelibc_r -ldcepthreads -lplishr_r -lc_r"
                #
		COMPILE="pli ${CLUser} ${CLFlags}"
                COMPILE="${COMPILE} ${CLCICS} ${CLLD}"
                COMPILE="${COMPILE} ${CLFile%.pli}.pli"
		;;
	esac

	case $CLType in
	e)	echo "${COMPILE}"
		return 0
		;;
	x)	${COMPILE}
		return $?
		;;
	f)	echo "${LINK}"
		return 0
		;;
	y)	${LINK}
		return $?
	esac
}

#------------------------------------------------------------------------
#
# SYNOPSIS: $CICS_LOG_LV_RAW_PATH $CICS_LOG_LV_BLOCK_PATH
#	    $CICS_SFS_LV_RAW_PATH $CICS_SFS_LV_BLOCK_PATH
#	    $CICS_PPCGWY_LV_RAW_PATH $CICS_PPCGWY_LV_BLOCK_PATH
#           $CICS_SIGTERM $CICS_SIGUSR1 $CICS_SIGHUP
# DESCRIPTION:
# This pair of shell parameters provide a platform indepenant location for
# the filesystem node where logical volumes are kept. All code wishing to
# check access etc. to an LV should use these variables as the path to the
# LV. Note that checking if an LV exists should be via cicslslv which
# performs sanity checks on the LV.
#
# The values associated with signals change from system to system. These
# variables provide the necessary mapping.

CICS_LOG_LV_RAW_PATH=/dev/r
CICS_LOG_LV_BLOCK_PATH=/dev/
CICS_SFS_LV_RAW_PATH=/dev/r
CICS_SFS_LV_BLOCK_PATH=/dev/
CICS_PPCGWY_LV_RAW_PATH=/dev/r
CICS_PPCGWY_LV_BLOCK_PATH=/dev/

CICS_SIGTERM=15
CICS_SIGUSR1=30
CICS_SIGHUP=1
CICS_SIGINT=2

#------------------------------------------------------------------------
#
# SYNOPSIS: CheckCanRun File ...
# DESCRIPTION:
# Checks caller can run all supplied program names
#
# This function allows the portability layer to map a check for execute
# permission on an emulation function into a check on the underlying program.
# i.e. a check for execution of lssrc is a direct check of the lssrc program
# on AIX, but on HPUX lssrc is a function which calls cicslssrc so we
# perform the execute check on cicslssrc.
#
# The addition of any emulation functions that are args to CheckCanRun
# calls will require an addition to the Location switch below.
#
CheckCanRun()
{
    typeset File
    typeset Location

    for File in $*
    do
	# whence returns the location of $File if it is a program file
	# on disk, otherwise it returns $File (as it's a function) or
	# nothing if the file is not any form of executable
	Location=$(whence $File)
	if [ -n "$Location" -a "${Location##*/}" = "$Location" ]
	then				# we have an emulation function name
	    case $Location in		# so map to underlying program
	    MkSSys)			# no underlying prog for these
		return 0 ;;		# so always executable
	    *)  print -u2 "ASSERT in $0: no lookup for $Location"
		exit 1 ;;
	    esac
	    Location=$(whence $File)
	fi

	if [ -z "$Location" ]
	then
	    return 1
	fi
    done
    return 0
}

#------------------------------------------------------------------------
#
# SYNOPSIS: ListLV Pathname
# DESCRIPTION:
# List the volume manager definition for a logical volume.
# 
ListLV()
{
    \lslv ${1##*/}		# lslv just needs LV name (without the path)
    return $?
}

#------------------------------------------------------------------------
#
# SYNOPSIS: GetSubSysStdOut <sub-system_name>
# DESCRIPTION:
# Obtain standard-output information for a specified subsystem.
#
GetSubSysStdOut()
{
    typeset DBEntry
    typeset Values
    typeset IFS=$IFS

    DBEntry=$(\odmget -q"subsysname=$1" SRCsubsys)
    if [ $? -ne 0 ]
    then
	echo "$DBEntry"
	return 1
    fi

    echo  "$DBEntry" | grep standout | sed 's/.*standout.*"\(.*\)"/\1/'
    return 0
 
}

#------------------------------------------------------------------------
#
# SYNOPSIS: GetRegionName <sub-system_name>
# DESCRIPTION:
# Obtain the CICS region name for a given SRC sub-system.
#
GetRegionName()
{

    echo "$1" | sed -e 's/^\(.*\)\.\(.*\)$/\2/'
    return 0
}

#------------------------------------------------------------------------
#
# SYNOPSIS: GetServerName <sub-system_name>
# DESCRIPTION:
# Obtain the log/sfs server name for a given SRC sub-system.
#
GetServerName()
{
    typeset DBEntry
    typeset Values
    typeset IFS=$IFS

    DBEntry=$(\odmget -q"subsysname=$1" SRCsubsys)
    if [ $? -ne 0 ]
    then
	print -- "$DBEntry"
	return 1
    fi

    print -- "$DBEntry" | sed -n '/cmdargs/s/.*"-n[ 	]*\(.*\)"/\1/p'
    return 0
 
}

#------------------------------------------------------------------------
#
# SYNOPSIS: MapLVol <server> <path_to_raw_device>
# DESCRIPTION:
# Associate AIX and Encina logical volumes, use the same name for both
# volumes, and a chunksize of 8. The chunksize does not seem to be
# critical so it is hard coded here.
#
MapLVol()
{
    typeset EncinaServer=$1
    typeset RawDisk=$2
    typeset EncinaLVol=${RawDisk##*/r}
    typeset AIXLVol=${RawDisk##*/r}
    typeset ChunkSize=64

    tkadmin map lvol -s $EncinaServer $EncinaLVol $AIXLVol $ChunkSize
    return $?
}

#------------------------------------------------------------------------
#
# SYNOPSIS: RgyEdit [<args>]
# DESCRIPTION:
# Map RgyEdit abstraction into real rgy_edit.
#
RgyEdit()
{
    \rgy_edit "$@"
    return $?
}

#-------------------------------------------------------------------------
#
# SYNOPSIS: RgyAddPrincipal [<args>]
# DESCRIPTION:
# No special processing is required for AIX, so we just massage the
# parameters into the proper form for rgy_edit and pass it all on. 
#
RgyAddPrincipal()
{

    typeset OUTPUT
    typeset RGYCMD
    typeset Ret
    typeset Rgy_flag

    while getopts :p:f:u: Rgy_flag
    do
        case $Rgy_flag in
	    p) RgyPrincipal="$OPTARG";;
	    f) RgyText="$OPTARG";;
	    u) RgyUser="$OPTARG";;
	    ?) print "Unrecognised option: $OPTARG";;
	    *) ;;
	esac
    done
    shift OPTIND-1

    if [ -z "$RgyPrincipal" ]
    then
	print "ASSERT in RgyAddPrincipal: principal name is required"
    else
	RGYCMD="add $RgyPrincipal"
        if [ -n "$RgyText" ]
	then 
	    RGYCMD="$RGYCMD -f \"$RgyText\""
	fi
        OUTPUT=$(RgyEdit -p -update << ENDFILE
	$RGYCMD
	quit
ENDFILE
	)				# run the command and keep o/p
    fi
    Ret=$?
    print -- "$OUTPUT"			# always need to send o/p back to user
    return $Ret				# return result to user
}

#-------------------------------------------------------------------------
#
# SYNOPSIS: MkSSys [<args>]
# DESCRIPTION:
# For AIX this call simply translates the alphabetic user information
# into its corresponding numeric uid. The real mkssys is then called 
# to perform processing. Perhaps equally importantly this layer ensures
# that mkssys function that we are not ready to make portable cannot be
# inadvertently introduced.
# All supplied parameters (with the exception of -u <name>) are simply
# remapped to the new command line.
#
alias mkssys=MkSSys
MkSSys()
{

typeset VALID_MKSSYS_OPTS=""		# mkssys supoported option list
typeset MKS_CMD=""			# initialise real mkssys command string
typeset MKS_PRM=""			# subsystem command parameters
typeset OUTPUT=""			# output capture for mkssys
typeset ret=				# return code from mkssys
typeset USERAID=""			# alphabetic user identifier
typeset USERNID=""			# numeric user identifier

#
# Parse options and arguments.
#

VALID_MKSSYS_OPTS=":p:s:u:a:i:o:e:ORdDqQKI:m:f:n:SE:G:w:"

while getopts $VALID_MKSSYS_OPTS mkssys_opt
do
   case $mkssys_opt in
      p)	MKS_CMD="$MKS_CMD -p $OPTARG";;
      s)	MKS_CMD="$MKS_CMD -s $OPTARG";;
      G)	MKS_CMD="$MKS_CMD -G $OPTARG";;
      u)	USERAID=$(lsuser -a id $OPTARG 2>&1)
		USERNID=$(expr "$USERAID" : '.*id=\([0-9]*\)')
		MKS_CMD="$MKS_CMD -u $USERNID";;
      a)	MKS_PRM="$OPTARG";;
      i)	MKS_CMD="$MKS_CMD -i $OPTARG";;
      o)	MKS_CMD="$MKS_CMD -o $OPTARG";;
      e)	MKS_CMD="$MKS_CMD -e $OPTARG";;
      d)	MKS_CMD="$MKS_CMD -d";;
      D)	MKS_CMD="$MKS_CMD -D";;
      q)	MKS_CMD="$MKS_CMD -q";;
      Q)	MKS_CMD="$MKS_CMD -Q";;
      f)	MKS_CMD="$MKS_CMD -f $OPTARG";;
      n)	MKS_CMD="$MKS_CMD -n $OPTARG";;
      S)	MKS_CMD="$MKS_CMD -S";;
      w)	MKS_CMD="$MKS_CMD -w $OPTARG";;
      O)	MKS_CMD="$MKS_CMD -O";;
      R)	MKS_CMD="$MKS_CMD -R";;
      E|t)	# Ignore nice value, synonym . . .
		;;
      K|I|m)	print -u2 "ASSERT MkSSys: unsupported -$mkssys_opt option.  Expected -S, subsys communication via signals."
		exit 1;;
      :)	print -u2 "ASSERT MkSSys: option $OPTARG requires an argument."
		exit 1;;
      \?)	print -u2 "ASSERT MkSSys: unknown option $OPTARG."
		exit 1;;
   esac
done

shift OPTIND-1
OUTPUT=$(\mkssys $MKS_CMD -a "$(eval print -- $MKS_PRM)")
ret=$?

print -- "$OUTPUT"
return $ret
}


#-------------------------------------------------------------------------
#
# SYNOPSIS: UpdateInittab
# DESCRIPTION:
# For AIX this call adds an entry to /etc/inittab to ensure that the
# cicsdd 3270 display subsystem and the CICS IPC kernel subroutines get added 
#correctly whenever the machine is restarted.

UpdateInittab()
{
rmitab rccics >/dev/null 2>&1

mkitab -i rc 'rccics:2:once:sh /usr/lpp/cics/etc/rc.cics > /dev/console 2>&1 # CICS 3270 and IPC subsystems'

return $?
}

#-------------------------------------------------------------------------
#
# SYNOPSIS: ConfigureKernelExtensions
# DESCRIPTION:
# For AIX this call calls the /usr/lpp/cics/etc/rc.cics shell script.

ConfigureKernelExtensions()
{
/usr/lpp/cics/etc/rc.cics

return 0
}

#-------------------------------------------------------------------------
#
# SYNOPSIS: $GETENCINAPATH
# DESCRIPTION:
# For AIX this sets up the encina path as /usr/lpp/encina/bin.

GETENCINAPATH=/usr/lpp/encina/bin

#------------------------------------------------------------------------

#-------------------------------------------------------------------------
#
# SYNOPSIS: $EGREPFLAGS
# DESCRIPTION:
# For AIX this sets up EGREPFLAGS as -s 

EGREPFLAGS=-s

#------------------------------------------------------------------------

#------------------------------------------------------------------------      
#                                                                             
#  CicsSafeLn
#                                                                             
# Perform ln safely without overwriting existing target files.
#                                                                             
#                                                                             
alias CicsSafeLn="ln"

#------------------------------------------------------------------------

#
# Module what string
#
#@(#)supos,23:06:24,Nov 11 2011,IBM WebSphere eXtended Transaction Runtime 1.0.0.1 xs100-L111111
#
